 /* spi-SCP1000D.c       VTI SCP1000 Pressure Sensor interface using spiasm library
  Function version with main() as demo 
  Uses pasm in xspiasm.spin, modified from SPI_Asm.spin propeller tool library object.
   T Montemarano  (MIT License)
   
   The SPI devices called in main() is a SCP1000 Pressure Sensor.    
        
 */
 
 #include "simpletools.h"                      // Library includes
 #include "spiasm.h"
   
 /* ******************************************************
    Uses SCP1000 Pressure Sensor as SPI device
 
        SCP1000 pin                  Prop pin 

          CSB              <-  Prop P13  low starts transfer, high terminates
          MISO            ->  Prop P12  Prop in (MPRE)
          MOSI            <-  Prop P11  Prop out (MFIRST)
          SCLK             <-  Prop P10  rise then fall  State = 0 
          DRDY            ->  Prop P14  Data is ready  
    SCP1000 SPI data rate = 500kHz max (clockd_ns1000 = 2000), try 2000 to start 
*/
 
 #define cstate1000 0             // 0 = start clock low, 1 = start clock high
 #define clockd_ns1000 2000   // Actual delay in nanoseconds (1000ns = 1 usec), not less than 300ns
                                           // ClockDelay, actual clock delay = 300ns + (cdelay-1) * 50ns
                                           // e.g. cdelay of 15 gives actual clock delay = 1 usec
                                           // cdelay = ((clockd_ns) - 250ns)) / 50ns
                                           // e.g. for 2 usec (i.e. 2000 ns): cdelay = 35
 //  ** Propeller pins 
 #define CSB1000 13
 #define MISO1000 12
 #define MOSI1000 11
 #define SCLK1000 10
 #define DRDY1000 14

// ******************************************************
/*  Function Prototypes are now in libspiasm

spia_t *start_SCP1000(int, int);    // clockd_ns1000, cstate1000
int init_SCP1000(spia_t *device, int, int, int, int, int);
int get_T_SCP1000(spia_t *device, int, int, int, int, int);  // function prototype for read Temperature - degC
int get_P_SCP1000(spia_t *device, int, int, int, int, int);  // function prototype for read Pressure - Pascals (Pa)
*/
spia_t *spia_1; // for opening additional SPI cogs add *spia_2, etc. Note multiple SPI devices can share same cog. 
   
int main()                              // Main function
{                           // get 4 readings and average them
  int ii;
  int Pa;                   // Pressure in Pascals
  int AvgPa;
  int degC;
  int AvgdegC;
  int degF;
  int mmHg;
// *** start and configure SPI devices         
  spia_1 = start_SCP1000(clockd_ns1000, cstate1000);

  int SCP1000_status = init_SCP1000(spia_1, MOSI1000, MISO1000, SCLK1000, CSB1000, DRDY1000);
  printi("Status Byte = %b \n", SCP1000_status);      // for debugging

  while(1)  // bb             primary loop  use low power triggered mode (2 Hz max rate)
  {
    AvgdegC = 0;
    AvgPa = 0;
    for(ii = 0; ii <= 3; ii++)   // aa     loop for SCP1000 data reads
    {
      degC = get_T_SCP1000(spia_1, MOSI1000, MISO1000, SCLK1000, CSB1000, DRDY1000);
      AvgdegC += degC; 
      Pa = get_P_SCP1000(spia_1, MOSI1000, MISO1000, SCLK1000, CSB1000, DRDY1000);     // read
      AvgPa += Pa; 
     pause(500);
     }              // end aa
// *** SCP1000 data handling & printout
     AvgdegC /= 4;
     AvgPa /= 4;
     degF = (AvgdegC * 90 / 5 + 320) / 10;
     mmHg = AvgPa /133;
     printi("Temperature = %d deg C, = %d deg F  \n Pressure = %d Pa  = %d mmHg\n\n", AvgdegC, degF, AvgPa, mmHg); 
     pause(20);

  }            // end bb      end primary loop
}              // end main

// *** Functions ***   Are now in libspiasm
